import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { CreatePersonalOfferDto } from './dto/create-personal-offer.dto';
import { PersonalOfferDto } from './dto/personal-offer.dto';
import { PersonalOffer, PersonalOfferDocument } from './schemas/personal-offer.schema';

@Injectable()
export class PersonalOffersService {
  constructor(@InjectModel(PersonalOffer.name) private personalOfferModel: Model<PersonalOfferDocument>) {}

  public async findOne(id: string): Promise<PersonalOffer> {
    const result: PersonalOfferDocument = await this.personalOfferModel.findById(id).exec();
    if (!result) {
      throw new HttpException('Personal offer does not exist', HttpStatus.NOT_FOUND);
    }
    return result;
  }

  public async create(createDto: CreatePersonalOfferDto): Promise<PersonalOfferDocument> {
    return this.personalOfferModel.create(createDto.personalOffer);
  }

  public async update(id: string, updatePersonalOfferDto: PersonalOfferDto): Promise<PersonalOfferDocument> {
    const result: PersonalOfferDocument = await this.personalOfferModel
      .findByIdAndUpdate(id, updatePersonalOfferDto)
      .exec();
    if (!result) {
      throw new HttpException('Invalid personal offer id for update', HttpStatus.BAD_REQUEST);
    }
    return result;
  }

  public async delete(id: string): Promise<PersonalOfferDocument> {
    const result: PersonalOfferDocument = await this.personalOfferModel.findByIdAndDelete(id).exec();
    if (!result) {
      throw new HttpException('Invalid personal offer id for deletion', HttpStatus.BAD_REQUEST);
    }
    return result;
  }
}
