import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { Type } from 'class-transformer';
import { ArrayNotEmpty, IsNotEmpty, ValidateNested } from 'class-validator';
import { Document, SchemaTypes, Types } from 'mongoose';
import { PersonalOfferDocument } from '../../personal-offers/schemas/personal-offer.schema';
import { Week } from '../../shared/schemas/week.schema';
import { StructureTypeDocument } from '../structure-type/structure-type.schema';
import { Address } from './address.schema';

export type StructureDocument = Structure & Document;

@Schema({ timestamps: { createdAt: 'createdAt', updatedAt: 'updatedAt' } })
export class Structure {
  constructor(data: Partial<Structure> = {}) {
    this.structureName = data.structureName;
    this.numero = data.numero;
    this.structureType = data.structureType;
    this.description = data.description;
    this.lockdownActivity = data.lockdownActivity;
    this.address = data.address;
    this.contactPhone = data.contactPhone;
    this.contactMail = data.contactMail;
    this.website = data.website;
    this.facebook = data.facebook;
    this.twitter = data.twitter;
    this.instagram = data.instagram;
    this.linkedin = data.linkedin;
    this.pmrAccess = data.pmrAccess;
    this.accessModality = data.accessModality;
    this.otherDescription = data.otherDescription;
    this.labelsQualifications = data.labelsQualifications;
    this.publics = data.publics;
    this.exceptionalClosures = data.exceptionalClosures;
    this.publicsAccompaniment = data.publicsAccompaniment;
    this.proceduresAccompaniment = data.proceduresAccompaniment;
    this.remoteAccompaniment = data.remoteAccompaniment;
    this.baseSkills = data.baseSkills;
    this.accessRight = data.accessRight;
    this.socialAndProfessional = data.socialAndProfessional;
    this.parentingHelp = data.parentingHelp;
    this.exceptionalClosures = data.exceptionalClosures;
    this.digitalCultureSecurity = data.digitalCultureSecurity;
    this.equipmentsAndServices = data.equipmentsAndServices;
    this.freeWorkShop = data.freeWorkShop;
    this.nbTablets = data.nbTablets;
    this.nbPrinters = data.nbPrinters;
    this.nbComputers = data.nbComputers;
    this.nbNumericTerminal = data.nbNumericTerminal;
    this.nbScanners = data.nbScanners;
    this.hours = data.hours;
    this.coord = data.coord;
    this.deletedAt = data.deletedAt;
    this.accountVerified = data.accountVerified;
    this.dataShareConsentDate = data.dataShareConsentDate;
    this.personalOffers = data.personalOffers;
  }

  @Prop()
  numero: string;

  @Prop()
  @IsNotEmpty()
  structureName: string;

  @Prop({ type: SchemaTypes.ObjectId, ref: 'StructureType' })
  @IsNotEmpty()
  structureType: StructureTypeDocument;

  @Prop()
  description: string;

  @Prop()
  lockdownActivity: string;

  @Prop()
  @ValidateNested({ each: true })
  @Type(() => Address)
  address: Address;

  @Prop()
  contactPhone?: string;

  @Prop()
  @IsNotEmpty()
  contactMail: string;

  @Prop()
  website: string;

  @Prop()
  facebook: string;

  @Prop()
  twitter: string;

  @Prop()
  instagram: string;

  @Prop()
  linkedin: string;

  @Prop()
  @IsNotEmpty()
  pmrAccess: boolean;

  @Prop()
  @ArrayNotEmpty()
  accessModality: string[];

  @Prop()
  otherDescription: string;

  @Prop()
  labelsQualifications: string[];

  @Prop()
  @ArrayNotEmpty()
  publics: string[];

  @Prop()
  exceptionalClosures: string;

  @Prop()
  publicsAccompaniment: string[];

  @Prop()
  proceduresAccompaniment: string[];

  @Prop()
  @IsNotEmpty()
  remoteAccompaniment: boolean;

  @Prop()
  baseSkills: string[];

  @Prop()
  accessRight: string[];

  @Prop()
  socialAndProfessional: string[];

  @Prop()
  parentingHelp: string[];

  @Prop()
  digitalCultureSecurity: string[];

  @Prop()
  equipmentsAndServices: string[];

  @Prop()
  @IsNotEmpty()
  freeWorkShop: boolean | string;

  @Prop()
  nbComputers: number;

  @Prop()
  @IsNotEmpty()
  nbPrinters: number;

  @Prop()
  @IsNotEmpty()
  nbTablets: number;

  @Prop()
  @IsNotEmpty()
  nbNumericTerminal: number;

  @Prop()
  @IsNotEmpty()
  nbScanners: number;

  @Prop()
  hours: Week;

  @Prop()
  coord: number[];

  @Prop()
  toBeDeletedAt?: Date;

  @Prop()
  deletedAt: Date;

  @Prop()
  accountVerified: boolean;

  @Prop()
  dataShareConsentDate: Date;

  @Prop({ type: [{ type: Types.ObjectId, ref: 'PersonalOffer' }] })
  personalOffers: PersonalOfferDocument[];

  @Prop()
  createdAt: Date;

  @Prop()
  updatedAt: Date;

  @Prop()
  idCNFS?: string;
}

export const StructureSchema = SchemaFactory.createForClass(Structure);
StructureSchema.index({ '$**': 'text' });
