import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { Types } from 'mongoose';
import { PersonalOfferDocument } from '../../personal-offers/schemas/personal-offer.schema';
import { Employer } from './employer.schema';
import { Job } from './job.schema';
import { UserRole } from '../enum/user-role.enum';

@Schema({ timestamps: true })
export class User {
  @Prop()
  createdAt: Date;

  @Prop({ required: true })
  email: string;

  @Prop({ required: true })
  name: string;

  @Prop({ required: true })
  surname: string;

  @Prop({ required: true })
  phone: string;

  @Prop({ required: true })
  password: string;

  @Prop({ default: false })
  emailVerified: boolean;

  @Prop({ default: null })
  validationToken: string;

  @Prop({ default: null })
  resetPasswordToken: string;

  @Prop({ enum: [UserRole.admin, UserRole.user], default: UserRole.user })
  role: number;

  @Prop({ default: null })
  changeEmailToken: string;

  @Prop({ default: null })
  newEmail: string;

  @Prop({ default: null })
  structuresLink: Types.ObjectId[];

  @Prop({ default: null })
  pendingStructuresLink: Types.ObjectId[];

  @Prop({ default: null })
  structureOutdatedMailSent: Types.ObjectId[];

  @Prop({ default: null })
  unattachedSince: Date;

  @Prop({ type: [{ type: Types.ObjectId, ref: 'PersonalOffer' }] })
  personalOffers: PersonalOfferDocument[];

  @Prop({ type: Types.ObjectId, ref: 'Employer' })
  employer: Employer;

  @Prop({ type: Types.ObjectId, ref: 'Job' })
  job: Job;

  @Prop({ required: false })
  description?: string;
}

export const UserSchema = SchemaFactory.createForClass(User);
UserSchema.index({ name: 'text', surname: 'text' });
