import { Test, TestingModule } from '@nestjs/testing';
import { multipleUsers } from '../../../test/mock/data/users.mock.data';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { SearchModule } from '../../search/search.module';
import { IUserRegistry } from '../interfaces/userRegistry.interface';
import { UserRegistrySearchService } from './userRegistry-search.service';

describe('UserRegistrySearchService Search cases', () => {
  let service: UserRegistrySearchService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [SearchModule, ConfigurationModule],
      providers: [UserRegistrySearchService],
    }).compile();

    service = module.get<UserRegistrySearchService>(UserRegistrySearchService);
    service['index'] = 'user-unit-test';
    // Init test cases
    await service.dropIndex();
    await service.createUserRegistryIndex();
    await Promise.all(multipleUsers.map((user: any) => service.indexUserRegistry(user)));

    // wait for the new structures to be indexed before search
    await service.refreshIndexUserRegistry();
    await new Promise((r) => setTimeout(r, 2000));
  }, 10000);

  it('should be defined', async () => {
    expect(service).toBeDefined();
  });
  describe('Search method', () => {
    it('should find Guilhem', async () => {
      const res = await service.search('Guilhem');
      expect(res[0].surname).toBe('Guilhem');
      expect(res.length).toBe(1);
    });
    it('should find adm', async () => {
      const res = await service.search('adm');
      expect(res[0].name).toBe('Admin');
      expect(res.length).toBe(1);
    });
    it('should find empty string', async () => {
      const res = await service.search('');
      expect(res.length).toBe(6);
    });
  });
  describe('Indexation methods', () => {
    it('should index User', async () => {
      const res = await Promise.all(multipleUsers.map((user: any) => service.indexUserRegistry(user)));
      expect(res).toBeTruthy();
      expect(res.length).toBe(6);
    });
    it('should update index', async () => {
      const res = await service.update(multipleUsers[0] as IUserRegistry);
      expect(res).toBeTruthy();
    });
    it('should delete index', async () => {
      const resAdm = await service.search('adm');
      const res = await service.deleteIndex(resAdm[0] as IUserRegistry);
      expect(res).toBeTruthy();
    });
  });
});
